@@ -6,7 +6,7 @@ from __future__ import division, with_statement
 Copyright 2011: Michael Hoffman <mmh1@uw.edu>
 Copyright 2009: Orion Buske <stasis@uw.edu>
 """
-__version__ = "$Revision: 322 $"
+__version__ = "$Revision: 339 $"
 
 import os
 import re
@@ -36,8 +36,8 @@ DTYPE_SEGMENT_END = int64
 DTYPE_SEGMENT_KEY = uint32
 DTYPE_STRAND = '|S1'
 
-class Annotations(object):
-    """Base class for representing annotations (BED/GFF/GTF files)
+class Annotation(object):
+    """Base class for representing annotation files (BED/GFF/GTF files)
 
     chromosomes: a dict
       key: chromosome name
@@ -51,7 +51,7 @@ class Annotations(object):
            This is the 4th column in a BED file and the 3rd column
            in GFF and GTF files.
 
-    filename: the filename from which the annotations were loaded
+    filename: the filename from which the annotation was loaded
 
     """
     class UnpickleError(Exception):
@@ -64,7 +64,7 @@ class Annotations(object):
         pass
 
     def __init__(self, filename, verbose=True):
-        """Returns an Annotations object derived from the given file
+        """Returns an Annotation object derived from the given file
 
         filename: path to a data file in one of the following formats:
           BED3+, GFF, GTF, or pickled Annotation. Format must be specified
@@ -95,7 +95,7 @@ class Annotations(object):
             if ext:
                 return ext[1:].lower()  # remove dot
 
-        raise Annotations.FilenameError("File missing appropriate extension:"
+        raise Annotation.FilenameError("File missing appropriate extension:"
                                         " %s" % filepath)
 
     def _load(self, filename, verbose=True):
@@ -254,7 +254,7 @@ class Annotations(object):
         self._inv_labels = label_dict
 
     def pickle(self, filename=None, verbose=True, clobber=False):
-        """Pickle the annotations into an output file"""
+        """Pickle the annotation into an output file"""
         import cPickle
         from .common import check_clobber, maybe_gzip_open
 
@@ -300,7 +300,7 @@ class Annotations(object):
         else:
             self._labels = labels
 
-class Segmentation(Annotations):
+class Segmentation(Annotation):
     """Representation of a segmentation
 
     Segments must be non-overlapping.
@@ -443,6 +443,8 @@ class RInterface(object):
         """Safer way to call R functions (without importing all that junk)
 
         None values are substituted with empty string
+        'ropts' keyword argument is parsed separately and should be None
+          or a list of 'param=value' strings to be passed to R.
         """
         self.start()
 
@@ -455,10 +457,17 @@ class RInterface(object):
             if arg is None:
                 args[arg_i] = ""
 
+
         for key, val in kwargs.iteritems():
             if val is None:
                 kwargs[key] = ""
 
+        ropts = kwargs.pop('ropts')
+        for opt in ropts:
+            if opt:
+                key, value = opt.split('=')
+                kwargs[key] = value
+
         # Save call to transcript
         if self._transcript:
             args_text = [self.arg_to_text(arg) for arg in args]
@@ -511,6 +520,46 @@ def open_transcript(dirpath, module, verified=False):
 
         yield res
 
+def add_common_options(parser, keys, **kwargs):
+    for key in keys:
+        if key == 'mnemonic_file':
+            parser.add_option("-m", "--mnemonic-file", dest="mnemonic_file",
+                              default=None, metavar="FILE",
+                              help="If specified, labels will be"
+                              " shown using mnemonics found in FILE")
+        elif key == 'outdir':
+            parser.add_option("-o", "--outdir", metavar="DIR",
+                              dest="outdir", default="%(MODULE)s" % kwargs,
+                              help="File output directory (will be created"
+                              " if it does not exist) [default: %default]")
+        elif key == 'clobber':
+            parser.add_option("--clobber", action="store_true",
+                              dest="clobber", default=False,
+                              help="Overwrite any existing output files.")
+        elif key == 'quiet':
+            parser.add_option("-q", "--quiet", action="store_false",
+                              dest="verbose", default=True,
+                              help="Do not print diagnostic messages.")
+        elif key == 'quick':
+            parser.add_option("--quick", action="store_true",
+                              dest="quick", default=False,
+                              help="Compute values only for one chromosome.")
+        elif key == 'replot':
+            parser.add_option("--replot", action="store_true",
+                              dest="replot", default=False,
+                              help="Load data from output tab files and"
+                              " regenerate plots instead of recomputing data.")
+        elif key == 'noplot':
+            parser.add_option("--noplot", action="store_true",
+                              dest="noplot", default=False,
+                              help="Do not generate any plots.")
+        elif key == 'ropts':
+            parser.add_option("-R", action="append", metavar="PARAM=VAL",
+                              dest="ropts", default=[],
+                              help="Pass VAL for PARAM when calling R"
+                              " functions. May be specified multiple times.")
+
+
 if __name__ == "__main__":
     pass
 
