@@ -1,19 +1,19 @@
 #!/usr/bin/env python
 
 """
-Given a SEGMENTATION and ANNOTATIONS file, 1) prints the distance of each
-segment to the nearest feature in the ANNOTATIONS file
+Given a SEGMENTATION and ANNOTATION file, 1) prints the distance of each
+segment to the nearest feature in the ANNOTATION file
 (zero if the two overlap) and
 2) generates a histogram of these distances.
 Distance is the difference between the nearest bases of the segment and the
-annotation, so if there is one base pair between them, the distance is 2.
-Distances can be strand-corrected with respect to stranded annotations
+feature, so if there is one base pair between them, the distance is 2.
+Distances can be strand-corrected with respect to stranded features
 by specifying --stranded.
 """
 
 from __future__ import division, with_statement
 
-__version__ = "$Revision: 321 $"
+__version__ = "$Revision: 339 $"
 
 
 import os
@@ -22,7 +22,8 @@ from time import time
 
 from numpy import NaN, zeros
 
-from . import Annotations, log, Segmentation, RInterface, open_transcript
+from . import Annotation, log, Segmentation, RInterface, open_transcript, \
+     add_common_options
 from .common import FeatureScanner, die, get_ordered_labels, \
     make_tabfilename, setup_directory, tab_saver
 from .html import save_html_div
@@ -67,7 +68,7 @@ def save_tab(counts, bins, labels, groups, outdir,
                                     str(bin), str(count)]))
                     saver.writerow(row)
 
-def save_plot(outdir, namebase=NAMEBASE, clobber=False,
+def save_plot(outdir, namebase=NAMEBASE, clobber=False, ropts=None,
               verbose=True, mnemonic_file=None, transcriptfile=None):
     R.start(transcriptfile, verbose=verbose)
     # Load data from corresponding tab file
@@ -76,7 +77,7 @@ def save_plot(outdir, namebase=NAMEBASE, clobber=False,
         die("Unable to find tab file: %s" % tabfilename)
 
     R.plot("save.distances", outdir, namebase, tabfilename,
-           mnemonic_file=mnemonic_file, clobber=clobber)
+           mnemonic_file=mnemonic_file, clobber=clobber, ropts=ropts)
 
 def save_html(dirpath, feature_file, mnemonic_file=None,
               clobber=False, verbose=True):
@@ -126,7 +127,7 @@ Failed test:
   correct:   %r
 """ % (n_bins, bin_width, distance, bins[index], index, answer)
 
-def print_distances(segmentation, annotations,
+def print_distances(segmentation, annotation,
                     n_bins=DEFAULT_BINS,
                     bin_width=DEFAULT_BIN_WIDTH,
                     verbose=True,
@@ -134,10 +135,10 @@ def print_distances(segmentation, annotations,
                     print_nearest=False,
                     all_overlapping=False):
     segment_data = segmentation.chromosomes
-    feature_data = annotations.chromosomes
+    feature_data = annotation.chromosomes
 
     labels = segmentation.labels
-    groups = annotations.labels
+    groups = annotation.labels
 
     print DELIM.join(PRINT_FIELDS)
 
@@ -212,20 +213,20 @@ def print_distances(segmentation, annotations,
     log("Distances calculated in %.1f seconds" % (time() - start_time), verbose)
     return counts, bin_distances
 
-def feature_distance(segment_file, annotations_file,
+def feature_distance(segment_file, annotation_file,
                      outdir, mnemonic_file=None,
                      n_bins=DEFAULT_BINS, bin_width=DEFAULT_BIN_WIDTH,
-                     clobber=False,
+                     clobber=False, ropts=None,
                      stranded=False, print_nearest=False,
                      all_overlapping=False,
                      verbose=True, replot=False, noplot=False):
     if not replot:
         setup_directory(outdir)
         segmentation = Segmentation(segment_file, verbose=verbose)
-        annotations = Annotations(annotations_file, verbose=verbose)
+        annotation = Annotation(annotation_file, verbose=verbose)
         labels = segmentation.labels
-        groups = annotations.labels
-        counts, bins = print_distances(segmentation, annotations,
+        groups = annotation.labels
+        counts, bins = print_distances(segmentation, annotation,
                                        n_bins=n_bins, bin_width=bin_width,
                                        verbose=verbose,
                                        stranded=stranded,
@@ -237,67 +238,55 @@ def feature_distance(segment_file, annotations_file,
     if not noplot:
         with open_transcript(outdir, MODULE) as transcriptfile:
             save_plot(outdir, clobber=clobber, verbose=verbose,
-                      mnemonic_file=mnemonic_file,
+                      mnemonic_file=mnemonic_file, ropts=ropts,
                       transcriptfile=transcriptfile)
 
-    save_html(outdir, annotations_file, mnemonic_file=mnemonic_file,
+    save_html(outdir, annotation_file, mnemonic_file=mnemonic_file,
               clobber=clobber, verbose=verbose)
 
 def parse_options(args):
     from optparse import OptionParser, OptionGroup
 
-    usage = "%prog [OPTIONS] SEGMENTATION ANNOTATIONS"
+    usage = "%prog [OPTIONS] SEGMENTATION ANNOTATION"
     version = "%%prog %s" % __version__
 
     parser = OptionParser(usage=usage, version=version,
                           description=__doc__.strip())
 
     group = OptionGroup(parser, "Basic options")
-    group.add_option("--clobber", action="store_true",
-                     dest="clobber", default=False,
-                     help="Overwrite existing output files if the specified"
-                     " directory already exists.")
-    group.add_option("-q", "--quiet", dest="verbose",
-                     default=True, action="store_false",
-                     help="Do not print diagnostic messages.")
+    add_common_options(group, ['clobber', 'quiet'])
     group.add_option("-s", "--stranded", dest="stranded",
                      default=False, action="store_true",
-                     help="Strand correct annotations in the ANNOTATIONS file."
-                     " If the annotations contains strand information,"
+                     help="Strand correct features in the ANNOTATION file."
+                     " If the feature contains strand information,"
                      " the sign of the distance value is used to portray the"
                      " relationship between the segment and the nearest"
-                     " stranded annotation. A negative distance value"
+                     " stranded feature. A negative distance value"
                      " indicates that the segment is nearest the 5' end of the"
-                     " annotation, whereas a positive value indicates the"
-                     " segment is nearest the 3' end of the annotation.")
+                     " feature, whereas a positive value indicates the"
+                     " segment is nearest the 3' end of the feature.")
     parser.add_option_group(group)
 
     group = OptionGroup(parser, "Print options")
     group.add_option("-p", "--print-nearest", dest="print_nearest",
                      default=False, action="store_true",
-                     help="The name of the nearest annotation will be printed"
+                     help="The name of the nearest feature will be printed"
                      " after each distance (with a space separating the"
-                     " two) for annotations from the ANNOTATIONS file."
+                     " two) for features from the ANNOTATION file."
                      " If multiple features are equally near (or overlap),"
                      " it is undefined which will be printed")
     group.add_option("-a", "--all-overlapping", dest="all_overlapping",
                      default=False, action="store_true",
-                     help="If multiple annotations in the ANNOTATIONS file"
+                     help="If multiple features in the ANNOTATION file"
                      " overlap a segment, a separate line is printed for each"
-                     " overlapping segment-annotation pair. This is most"
+                     " overlapping segment-feature pair. This is most"
                      " interesting with --print-nearest. Otherwise,"
                      " the first overlapping segment will be used for"
                      " printing.")
     parser.add_option_group(group)
 
     group = OptionGroup(parser, "Plotting options")
-    group.add_option("--noplot", action="store_true",
-                     dest="noplot", default=False,
-                     help="Do not generate plots")
-    group.add_option("--replot", action="store_true",
-                     dest="replot", default=False,
-                     help="Load data from output tab files and"
-                     " regenerate plots instead of recomputing data")
+    add_common_options(group, ['noplot', 'replot'])
     group.add_option("-n", "--n-bins", metavar="N", type="int",
                      dest="n_bins", default=DEFAULT_BINS,
                      help="Number of bins to use in histogram for distances"
@@ -316,14 +305,11 @@ def parse_options(args):
     parser.add_option_group(group)
 
     group = OptionGroup(parser, "Output options")
-    group.add_option("-m", "--mnemonic-file", dest="mnemonic_file",
-                     default=None, metavar="FILE",
-                     help="If specified, labels will be shown using"
-                     " mnemonics found in FILE")
-    group.add_option("-o", "--outdir", metavar="DIR",
-                     dest="outdir", default="%s" % MODULE,
-                     help="File output directory (will be created"
-                     " if it does not exist) [default: %default]")
+    add_common_options(group, ['mnemonic_file', 'outdir'], MODULE=MODULE)
+    parser.add_option_group(group)
+
+    group = OptionGroup(parser, "R options")
+    add_common_options(group, ['ropts'])
     parser.add_option_group(group)
 
     (options, args) = parser.parse_args(args)
