@@ -1,11 +1,11 @@
 #!/usr/bin/env python
 from __future__ import division, with_statement
 
-__version__ = "$Revision: 321 $"
+__version__ = "$Revision: 339 $"
 
 """
 Provides command-line and package entry points for analyzing nucleotide
-and dinucleotide frequencies for each label in the segmentation or annotations
+and dinucleotide frequencies for each label in the segmentation or annotation
 file.
 
 """
@@ -17,9 +17,10 @@ import warnings
 
 from collections import defaultdict
 from genomedata import Genome
-from numpy import zeros
+from numpy import zeros, bincount, array
 
-from . import log, Annotations, die, RInterface
+from . import log, Annotation, die, RInterface, open_transcript, \
+     add_common_options
 from .common import make_tabfilename, setup_directory, tab_saver
 
 from .html import save_html_div
@@ -52,9 +53,19 @@ DINUC_CATEGORIES = [[('A', 'A'), ('T', 'T')],
 
 R = RInterface(["common.R", "dinucleotide.R"])
 
+def reduce_nucs(arr):
+    # Return array with nucs compressed to 0-4
+    return numpy.digitize(arr, [ord(i) for i in 'CGNT'])
+
+def zip_nucs(nuc1, nuc2):
+    # Given 2 reduced nuc arrays, return the dinuc value where dinucs are
+    # formed from zip(nuc1, nuc2)
+    # each reduced nuc array has a range of 0-4
+    return nuc1 * 5 + nuc2
+
 ## Caclulates nucleotide and dinucleotide frequencies over the specified
-## annotations
-def calc_nucleotide_frequencies(annotations, genome,
+## annotation data
+def calc_nucleotide_frequencies(annotation, genome,
                                 nuc_categories=NUC_CATEGORIES,
                                 dinuc_categories=DINUC_CATEGORIES,
                                 quick=False, verbose=True):
@@ -64,72 +75,89 @@ def calc_nucleotide_frequencies(annotations, genome,
     quick_nuc_categories = defaultdict(dict)
     for index, category in enumerate(nuc_categories):
         for entry in category:
-            #print("Mapping %s -> %d" % (entry, index))
-            quick_nuc_categories[entry] = index
+            key = reduce_nucs(array([ord(entry)], dtype='i8'))[0]
+            #print("Mapping %s -> %d -> %d" % (entry, key, index))
+            assert key not in quick_nuc_categories, \
+                   "Error: nucleotide hash for %s is not unique" % entry
+            quick_nuc_categories[key] = index
+    quick_nuc_categories = dict(quick_nuc_categories)
 
     quick_dinuc_categories = defaultdict(dict)
     for index, category in enumerate(dinuc_categories):
         for entry in category:
-            #print("Mapping %s%s -> %d" % (entry[0], entry[1], index))
-            quick_dinuc_categories[entry] = index
+            # Second letter between 0-4, first letter (0-4)*5
+            nucs = reduce_nucs(array([ord(i) for i in entry], dtype='i8'))
+            key = zip_nucs(nucs[0], nucs[1])
+            #print("Mapping %s%s -> %d -> %d" % (entry[0], entry[1], key, index))
+            assert key not in quick_dinuc_categories, \
+                   "Error: dinucleotide hash for %s%s is not unique" % \
+                   (entry[0], entry[1])
+            quick_dinuc_categories[key] = index
+    quick_dinuc_categories = dict(quick_dinuc_categories)
 
     # Store counts of each (di)nucleotide observed
     # separated by label
     nuc_counts = defaultdict(dict)
     dinuc_counts = defaultdict(dict)
 
-    labels = annotations.labels
+    labels = annotation.labels
     for label_key in labels.iterkeys():
         nuc_counts[label_key] = zeros(len(nuc_categories)+1, dtype=numpy.long)
         dinuc_counts[label_key] = zeros(len(dinuc_categories)+1,
                                         dtype=numpy.long)
 
-    # Count (di)nucleotides over annotations
+    # Count (di)nucleotides over annotation
     for chromosome in genome:
         chrom = chromosome.name
 
         try:
-            chrom_rows = annotations.chromosomes[chrom]
-            # Store entire chromosome's sequence as string in memory
-            # for speedup
+            chrom_rows = annotation.chromosomes[chrom]
+            # Store entire chromosome's sequence as string in memory for speed
             warnings.simplefilter("ignore")
-            chrom_sequence = chromosome.seq[0:chromosome.end].tostring().upper()
+            log("  %s" % chrom, verbose)
+            log("    preparing chromosome sequence...", verbose)
+            chrom_sequence = chromosome.seq[0:chromosome.end]
+            # Convert to capital letter within integer representation
+            chrom_sequence[chrom_sequence >= ord('a')] -= ord('a') - ord('A')
+            # Convert to reduced integer representation (0-4)
+            chrom_sequence = reduce_nucs(chrom_sequence)
             warnings.resetwarnings()
         except KeyError:
             continue
 
-        log("  %s" % chrom, verbose)
+        log("    counting nucs and dinucs...", verbose)
+        interval = int(len(chrom_rows) / 70)
+        row_i = 0
         for row in chrom_rows:
+            if row_i % interval == 0:
+                n_done = int(row_i / interval)
+                n_left = 70 - n_done
+                log("    %s%s" % ('=' * n_done, '-' * n_left), verbose)
+
             start = row['start']
             end = row['end']
             label = row['key']
 
+            lab_nuc_counts = nuc_counts[label]
+            lab_dinuc_counts = dinuc_counts[label]
             sequence = chrom_sequence[start:end]
+            # Count nucs
+            val_counts = bincount(sequence)
+            for val in val_counts.nonzero()[0]:
+                n = val_counts[val]
+                # Put in last bin if not found
+                index = quick_nuc_categories.get(val, len(lab_nuc_counts) - 1)
+                lab_nuc_counts[index] += n
 
-            # XXXopt: could be optimized to use matrix operations
-            # to determine number of occurances of each (di)nuc
-            # Inch through iteration to look at pairs efficiently
-            cur_nuc = None
-            prev_nuc = None
-            for nuc in sequence:
-                prev_nuc = cur_nuc
-                cur_nuc = nuc
+            # Count dinucs efficiently
+            if len(sequence) > 1:
+                val_counts = bincount(zip_nucs(sequence[:-1], sequence[1:]))
+                for val in val_counts.nonzero()[0]:
+                    n = val_counts[val]
+                    index = quick_dinuc_categories.get(val, len(lab_dinuc_counts) - 1)
+                    dinuc_counts[label][index] += n
 
-                # Record nucleotide
-                nuc_cat = quick_nuc_categories[cur_nuc]
-                if nuc_cat == {}:  # Didn't find; put in last bin
-                    #print >>sys.stderr, "Unknown nuc: %s" % (cur_nuc)
-                    nuc_cat = len(nuc_counts[label]) - 1
-
-                nuc_counts[label][nuc_cat] += 1
-
-                # Record dinucleotide (last and current)
-                if prev_nuc is not None:
-                    dinuc_cat = quick_dinuc_categories[(prev_nuc, cur_nuc)]
-                    if dinuc_cat == {}:  # Didn't find; put in last bin
-                        dinuc_cat = len(dinuc_counts[label]) - 1
-
-                    dinuc_counts[label][dinuc_cat] += 1
+            row_i += 1
 
         # Only look at first chromosome if quick
         if quick: break
@@ -163,16 +191,16 @@ def save_tab(labels, nuc_counts, dinuc_counts, dirpath,
                            dinuc_counts[label_key])
             saver.writerow(row)
 
-def save_plot(dirpath, clobber=False, verbose=True,
-              mnemonic_file="", namebase=NAMEBASE):
-    R.start(verbose=verbose)
+def save_plot(dirpath, clobber=False, verbose=True, ropts=None,
+              mnemonic_file="", namebase=NAMEBASE, transcriptfile=None):
+    R.start(verbose=verbose, transcriptfile=transcriptfile)
 
     tabfilename = make_tabfilename(dirpath, NAMEBASE)
     if not os.path.isfile(tabfilename):
         die("Unable to find tab file: %s" % tabfilename)
 
     R.plot("save.dinuc", dirpath, namebase, tabfilename,
-           mnemonic_file=mnemonic_file, clobber=clobber)
+           mnemonic_file=mnemonic_file, clobber=clobber, ropts=ropts)
 
 def save_html(dirpath, clobber=False, verbose=True, mnemonicfile=None):
     save_html_div(HTML_TEMPLATE_FILENAME, dirpath, NAMEBASE, clobber=clobber,
@@ -181,23 +209,26 @@ def save_html(dirpath, clobber=False, verbose=True, mnemonicfile=None):
 
 ## Package entry point
 def validate(filename, genomedatadir, dirpath, clobber=False, quick=False,
-             replot=False, noplot=False, mnemonic_file=None, verbose=True):
+             replot=False, noplot=False, mnemonic_file=None, verbose=True,
+             ropts=None):
     setup_directory(dirpath)
     if not replot:
-        annotations = Annotations(filename, verbose=verbose)
-        labels = annotations.labels
+        annotation = Annotation(filename, verbose=verbose)
+        labels = annotation.labels
 
         with Genome(genomedatadir) as genome:
             nuc_counts, dinuc_counts = \
-                calc_nucleotide_frequencies(annotations, genome,
+                calc_nucleotide_frequencies(annotation, genome,
                                             quick=quick, verbose=verbose)
 
         save_tab(labels, nuc_counts, dinuc_counts, dirpath,
                  clobber=clobber, verbose=verbose)
 
     if not noplot:
-        save_plot(dirpath, clobber=clobber, verbose=verbose,
-                  mnemonic_file=mnemonic_file)
+        with open_transcript(dirpath, MODULE) as transcriptfile:
+            save_plot(dirpath, clobber=clobber, verbose=verbose,
+                      mnemonic_file=mnemonic_file, ropts=ropts,
+                      transcriptfile=transcriptfile)
 
     save_html(dirpath, clobber=clobber, mnemonicfile=mnemonic_file,
               verbose=verbose)
@@ -205,39 +236,21 @@ def validate(filename, genomedatadir, dirpath, clobber=False, quick=False,
 def parse_options(args):
     from optparse import OptionGroup, OptionParser
 
-    usage = "%prog [OPTIONS] ANNOTATIONS GENOMEDATAFILE"
+    usage = "%prog [OPTIONS] ANNOTATION GENOMEDATAFILE"
     version = "%%prog %s" % __version__
     parser = OptionParser(usage=usage, version=version)
 
     group = OptionGroup(parser, "Flags")
-    group.add_option("--clobber", action="store_true",
-                     dest="clobber", default=False,
-                     help="Overwrite existing output files if the specified"
-                     " directory already exists.")
-    group.add_option("-q", "--quiet", action="store_false",
-                     dest="verbose", default=True,
-                     help="Do not print diagnostic messages.")
-    group.add_option("--quick", action="store_true",
-                     dest="quick", default=False,
-                     help="Compute values only for one chromosome.")
-    group.add_option("--replot", action="store_true",
-                     dest="replot", default=False,
-                     help="Load data from output tab files and"
-                     " regenerate plots instead of recomputing data")
-    group.add_option("--noplot", action="store_true",
-                     dest="noplot", default=False,
-                     help="Do not generate plots")
+    add_common_options(group, ['clobber', 'quiet', 'quick', 'replot',
+                               'noplot'])
     parser.add_option_group(group)
 
     group = OptionGroup(parser, "Output")
-    group.add_option("-m", "--mnemonic-file", dest="mnemonic_file",
-                     default=None, metavar="FILE",
-                     help="If specified, labels will be shown using"
-                     " mnemonics found in FILE.")
-    group.add_option("-o", "--outdir",
-                     dest="outdir", default="%s" % MODULE, metavar="DIR",
-                     help="File output directory (will be created"
-                     " if it does not exist) [default: %default]")
+    add_common_options(group, ['mnemonic_file', 'outdir'], MODULE=MODULE)
+    parser.add_option_group(group)
+
+    group = OptionGroup(parser, "R options")
+    add_common_options(group, ['ropts'])
     parser.add_option_group(group)
 
     (options, args) = parser.parse_args(args)
