@@ -11,7 +11,7 @@ distribution over tracks and labels.
 # Must be the same as the folder containing this script
 MODULE="signal_distribution"
 
-__version__ = "$Revision: 321 $"
+__version__ = "$Revision: 339 $"
 
 
 import os
@@ -21,10 +21,10 @@ from collections import defaultdict
 from genomedata import Genome
 from itertools import repeat
 from functools import partial
-from numpy import array, ceil, compress, floor, fromiter, histogram, \
-     isfinite, NAN, nanmax, nanmin, nansum, NINF, PINF
+from numpy import apply_along_axis, array, ceil, compress, floor, fromiter, \
+    histogram, isfinite, NAN, nanmax, nanmin, nansum, NINF, PINF, square, zeros
 
-from . import log, Segmentation, die, RInterface
+from . import log, Segmentation, die, RInterface, add_common_options
 from .common import iter_segments_continuous, iter_supercontig_segments, \
      make_tabfilename, setup_directory, tab_reader, tab_saver
 from .html import save_html_div
@@ -95,8 +95,6 @@ class SignalHistogram(object):
 
         labels = segmentation.labels
 
-        label_bins = xrange(0, len(labels) + 1)  # [0, 1, ..., n, n + 1]
-
         tracks = genome.tracknames_continuous
         max_bins = ceil(genome.maxs).astype(int)
         min_bins = floor(genome.mins).astype(int)
@@ -181,6 +179,86 @@ class SignalHistogram(object):
 
         return SignalHistogram(res, nseg_dps=nseg_dps)
 
+    @staticmethod
+    def calculate_stat(genome, segmentation, chroms=None,
+                       quick=False, verbose=True):
+        """
+        Computes a mean and variance for each track-label pair.
+        
+
+        if chroms is:
+        - a sequence of chromosome names: only those chromosomes are processed
+        - False, [], None: all chromosomes are processed
+
+        if calc_ranges is:
+        - False: use the precomputed ranges stored for the entire genome file
+        - True: calculate the segmentation ranges across the entire input
+ 
+        if quick is:
+        - True: the histogram is calculated for only the first chromosome
+ 
+        returns a dict
+ 
+        """
+        assert genome is not None
+        assert segmentation is not None
+ 
+        labels = segmentation.labels
+        
+        tracks = genome.tracknames_continuous
+        max_bins = ceil(genome.maxs).astype(int)
+        min_bins = floor(genome.mins).astype(int)
+        # A dict from tracks to a range tuple
+        track_indices = dict(zip(tracks, range(len(tracks))))
+        
+        if len(tracks) == 0:
+            die("Trying to calculate histogram for no tracks")
+            
+        nseg_dps = 0  # Number of non-NaN data points in segmentation tracks
+        sum_total, sum2_total, nseg_dps_total = zeros(\
+            (len(tracks), len(labels)), dtype=float),  
+        zeros((len(tracks), len(labels)), dtype=float), 
+        zeros((len(tracks), len(labels)), dtype=float)
+        log("Generating signal distribution histograms", verbose)
+ 
+        with genome:
+            if chroms:
+                chromosomes = [genome[chrom] for chrom in chroms]
+            else:
+                chromosomes = genome
+                
+            for chromosome in chromosomes:
+                # Iterate through supercontigs and segments together
+                for segment, continuous_seg in \
+                        iter_segments_continuous(chromosome, segmentation,
+                                                 verbose=verbose):
+ 
+                    sum_segment, sum2_segment, nseg_dps_segment  = \
+                        apply_along_axis(get_stat, 0, continuous_seg)
+                    seg_label = labels[segment['key']]
+                    sum_total[:,seg_label], \
+                        sum2_total[:,seg_label], \
+                        nseg_dps_total[:,seg_label] = \
+                        sum_total[:,seg_label] + sum_segment, \
+                        sum2_total[:,seg_label] + sum2_segment, \
+                        nseg_dps_total[:,seg_label] + nseg_dps_segment
+                     # Iterate through each track
+ 
+                if quick: break  # 1 chromosome
+ 
+        mean = sum_total/nseg_dps_total
+        sd = (sum2_total -
+              ((sum_total**2)/nseg_dps_total))/(nseg_dps_total -1)
+        nseg_dps = sum(nseg_dps_total)
+        stats = defaultdict(partial(defaultdict, dict))
+        for trackname, track_index in track_indices.iteritems():
+            for label in labels:
+                cur_stat = stats[label][trackname]
+                cur_stat["sd"] = sd[track_index,label]
+                cur_stat["mean"] = mean[track_index,label]
+        return stats, nseg_dps 
+ 
+
     @staticmethod
     def calculate2(genome, segmentation, nbins=None, chroms=None,
                   calc_ranges=False,
@@ -190,8 +268,6 @@ class SignalHistogram(object):
 
         labels = segmentation.labels
 
-        label_bins = xrange(0, len(labels) + 1)  # [0, 1, ..., n, n + 1]
-
         tracks = genome.tracknames_continuous
         max_bins = ceil(genome.maxs).astype(int)
         min_bins = floor(genome.mins).astype(int)
@@ -370,12 +446,27 @@ def save_stats_tab(stats, dirpath, clobber=False, verbose=True,
                 sd = track_stats["sd"]
                 saver.writerow(locals())
 
+def save_stats_tab_genhist(stats, dirpath, clobber=False, verbose=True,
+                           namebase=NAMEBASE_STATS, 
+                           fieldnames=FIELDNAMES_STATS):
+    with tab_saver(dirpath, namebase, fieldnames, verbose=verbose,
+                   clobber=clobber) as saver:
+        for label, label_stats in stats.iteritems():
+            for trackname, track_stats in label_stats.iteritems():
+                mean = track_stats["mean"]
+                sd = track_stats["sd"]
+                saver.writerow(locals())
+
 def constant_factory(val):
     return repeat(val).next
 
 def remove_nans(numbers):
     return numbers[isfinite(numbers)]
 
+def get_stat(arr):
+    arr_nonan = remove_nans(arr)
+    return arr_nonan.sum(), square(arr_nonan).sum(), arr_nonan.shape[0]
+
 ## Returns a dict of trackname -> tuples: (min, max), one for each trackname
 def seg_min_max(chromosome, segmentation, verbose=False):
     #print >>sys.stderr, "Looking for min & max in chromosome ", \
@@ -610,7 +701,7 @@ def load_track_ranges(genome, segmentation=None):
 def save_stats_plot(dirpath, namebase=NAMEBASE_STATS, filename=None,
                     clobber=False, mnemonic_file=None, translation_file=None,
                     allow_regex=False, gmtk=False, verbose=True,
-                    label_order=[], track_order=[]):
+                    label_order=[], track_order=[], ropts=None):
     """
     if filename is specified, it overrides dirpath/namebase.tab as
     the data file for plotting.
@@ -628,7 +719,7 @@ def save_stats_plot(dirpath, namebase=NAMEBASE_STATS, filename=None,
 
     R.plot("save.track.stats", dirpath, namebase, filename,
            mnemonic_file=mnemonic_file,
-           translation_file=translation_file,
+           translation_file=translation_file, ropts=ropts,
            as_regex=allow_regex, gmtk=gmtk, clobber=clobber,
            label_order=label_order, track_order=track_order)
 
@@ -670,11 +761,11 @@ def validate(bedfilename, genomedatadir, dirpath,
              clobber=False, calc_ranges=False, inputdirs=None,
              quick=False, replot=False, noplot=False, verbose=True,
              nbins=NBINS, value_range=(None, None),
-             ecdf=False, mnemonic_file=None,
-             create_mnemonics=False, chroms=None,
+             ecdf=False, mnemonic_file=None, genhist=False,
+             create_mnemonics=False, chroms=None, ropts=None,
              label_order_file=None, track_order_file=None):
 
-    if not replot:
+    if not genhist or not replot:
         setup_directory(dirpath)
         genome = Genome(genomedatadir)
         segmentation = Segmentation(bedfilename, verbose=verbose)
@@ -686,30 +777,44 @@ def validate(bedfilename, genomedatadir, dirpath,
         nlabels = len(labels)
         fieldnames = FIELDNAMES
 
-    if inputdirs:
-        histogram = SignalHistogram()
-        for inputdir in inputdirs:
-            try:
-                sub_histogram = SignalHistogram.read(inputdir, verbose=verbose)
-            except IOError, e:
-                log("Problem reading data from %s: %s" % (inputdir, e))
-            else:
-                histogram.add(sub_histogram)
 
-    elif not replot:
-        # Generate histogram
-        histogram = SignalHistogram.calculate(genome, segmentation,
-                                              nbins=nbins, verbose=verbose,
-                                              calc_ranges=calc_ranges,
-                                              value_range=value_range,
-                                              quick=quick, chroms=chroms)
+    if genhist:
+        if inputdirs:
+            histogram = SignalHistogram()
+            for inputdir in inputdirs:
+                try:
+                    sub_histogram = SignalHistogram.read(inputdir, verbose=verbose)
+                except IOError, e:
+                    log("Problem reading data from %s: %s" % (inputdir, e))
+                else:
+                    histogram.add(sub_histogram)
+
+        elif not replot:
+            # Generate histogram
+            histogram = SignalHistogram.calculate(genome, segmentation,
+                                                  nbins=nbins, verbose=verbose,
+                                                  calc_ranges=calc_ranges,
+                                                  value_range=value_range,
+                                                  quick=quick, chroms=chroms)
+        else:
+            histogram = SignalHistogram.read(dirpath, verbose=verbose)
+
+        if not replot:
+            histogram.save(dirpath, clobber=clobber, verbose=verbose)
+
+            stats = calc_stats(histogram)
+            save_stats_tab(stats, dirpath, clobber=clobber, verbose=verbose,
+                           namebase=NAMEBASE_STATS)
+
+            if mnemonic_file is None and create_mnemonics:
+                statsfilename = make_tabfilename(dirpath, NAMEBASE_STATS)
+                mnemonic_file = create_mnemonic_file(statsfilename, dirpath,
+                                                     clobber=clobber,
+                                                     verbose=verbose)
     else:
-        histogram = SignalHistogram.read(dirpath, verbose=verbose)
+        stats, nseg_dps = SignalHistogram.calculate_stat(\
+            genome, segmentation, quick=quick, chroms=chroms)
 
-    if not replot:
-        histogram.save(dirpath, clobber=clobber, verbose=verbose)
-
-        stats = calc_stats(histogram)
         save_stats_tab(stats, dirpath, clobber=clobber, verbose=verbose,
                        namebase=NAMEBASE_STATS)
 
@@ -719,6 +824,7 @@ def validate(bedfilename, genomedatadir, dirpath,
                                                  clobber=clobber,
                                                  verbose=verbose)
 
+ 
     if not noplot:
         if label_order_file is not None:
             log("Reading label ordering from: %s" % label_order_file)
@@ -730,13 +836,16 @@ def validate(bedfilename, genomedatadir, dirpath,
 
         save_stats_plot(dirpath, namebase=NAMEBASE_STATS, clobber=clobber,
                         mnemonic_file=mnemonic_file, verbose=verbose,
-                        label_order=label_order, track_order=track_order)
+                        label_order=label_order, track_order=track_order,
+                        ropts=ropts)
 
-    if histogram:
+    if genhist and histogram:
         try:
             nseg_dps = histogram.metadata["nseg_dps"]
         except KeyError:
             nseg_dps = None
+
+    if not genhist or histogram:
         save_html(dirpath, genomedatadir, nseg_dps=nseg_dps,
                   ecdf=ecdf, clobber=clobber, verbose=verbose)
 
@@ -749,23 +858,11 @@ def parse_options(args):
     parser = OptionParser(usage=usage, version=version)
 
     group = OptionGroup(parser, "Flags")
-    group.add_option("--clobber", action="store_true",
-                     dest="clobber", default=False,
-                     help="Overwrite existing output files if the specified"
-                     " directory already exists.")
-    group.add_option("-q", "--quiet", action="store_false",
-                     dest="verbose", default=True,
-                     help="Do not print diagnostic messages.")
-    group.add_option("--quick", action="store_true",
-                     dest="quick", default=False,
-                     help="Compute values only for one chromosome.")
-    group.add_option("--noplot", action="store_true",
-                     dest="noplot", default=False,
-                     help="Do not generate plots")
-    group.add_option("--replot", action="store_true",
-                     dest="replot", default=False,
-                     help="Load data from output tab files and"
-                     " regenerate plots instead of recomputing data")
+    add_common_options(group, ['clobber', 'quiet', 'quick', 'noplot',
+                               'replot'])
+    group.add_option("--genhist", action="store_true",
+                     dest="genhist", default=False,
+                     help="Generate histogram for each track")
     group.add_option("--create-mnemonics", action="store_true",
                      dest="create_mnemonics", default=False,
                      help="If mnemonics are not specified, they will be"
@@ -792,23 +889,9 @@ def parse_options(args):
                      " where CHROM is a chromosome name such as chr21 or"
                      " chrX (option can be used multiple times to allow"
                      " multiple chromosomes)")
-#     group.add_option("--min-value", type="float", metavar="N",
-#                      dest="min_value", default=None,
-#                      help="Minimum signal track value used in binning"
-#                      " (overrides min from --calc-ranges)"
-#                      " (values below will be ignored)")
-#     group.add_option("--max-value", type="float", metavar="N",
-#                      dest="max_value", default=None,
-#                      help="Maximum signal track value used in binning"
-#                      " (overrides max from --calc-ranges)"
-#                      " (values above will be ignored)")
     parser.add_option_group(group)
 
     group = OptionGroup(parser, "I/O options")
-    group.add_option("-m", "--mnemonic-file", dest="mnemonic_file",
-                     default=None, metavar="FILE",
-                     help="If specified, labels will be shown using"
-                     " mnemonics found in FILE")
     group.add_option("--order-tracks", dest="track_order_file",
                      default=None, metavar="FILE",
                      help="If specified, tracks will be displayed"
@@ -829,10 +912,11 @@ def parse_options(args):
                      " directory of a previous run of this module."
                      " This option can be specified multiple times to"
                      " merge previous results together.")
-    group.add_option("-o", "--outdir", metavar="DIR",
-                     dest="outdir", default="%s" % MODULE,
-                     help="File output directory (will be created"
-                     " if it does not exist) [default: %default]")
+    add_common_options(group, ['mnemonic_file', 'outdir'], MODULE=MODULE)
+    parser.add_option_group(group)
+
+    group = OptionGroup(parser, "R options")
+    add_common_options(group, ['ropts'])
     parser.add_option_group(group)
 
     (options, args) = parser.parse_args(args)
@@ -852,23 +936,10 @@ def main(args=sys.argv[1:]):
     (options, args) = parse_options(args)
     bedfilename = args[0]
     genomedatadir = args[1]
-    kwargs = {"clobber": options.clobber,
-              "verbose": options.verbose,
-              "quick": options.quick,
-              "calc_ranges": options.calc_ranges,
-              "replot": options.replot,
-              "noplot": options.noplot,
-              "nbins": options.nbins,
-#               "value_range": (options.min_value, options.max_value),
-              "ecdf": options.ecdf,
-              "inputdirs": options.inputdirs,
-              "mnemonic_file": options.mnemonic_file,
-              "create_mnemonics": options.create_mnemonics,
-              "chroms": options.chroms,
-              "label_order_file": options.label_order_file,
-              "track_order_file": options.track_order_file}
 
-    validate(bedfilename, genomedatadir, options.outdir, **kwargs)
+    kwargs = dict(options.__dict__)
+    outdir = kwargs.pop('outdir')
+    validate(bedfilename, genomedatadir, outdir, **kwargs)
 
 if __name__ == "__main__":
     sys.exit(main())
