@@ -1,3 +1,6 @@
+#!/usr/bin/env python
+from __future__ import with_statement
+
 import inspect
 import sys
 import unittest
@@ -5,8 +8,7 @@ import unittest
 from numpy import array
 from tempfile import NamedTemporaryFile
 
-from segtools import Segmentation
-from segtools.common import load_features
+from segtools import Annotations, Segmentation
 from segtools.overlap import calc_overlap
 
 def data2bed(lines):
@@ -37,22 +39,19 @@ class OverlapTester(unittest.TestCase):
         self.features = []
         self._open_files = []
         for type, data in [self.subject, self.query]:
-            new_file = NamedTemporaryFile()
-            self._open_files.append(new_file)
-            features = None
             if type == "bed":
+                new_file = NamedTemporaryFile(suffix=".bed")
                 new_file.write(data2bed(data))
-                new_file.flush()
-                features = Segmentation(new_file.name, verbose=False)
             elif type == "gff":
+                new_file = NamedTemporaryFile(suffix=".gff")
                 new_file.write(data2gff(data))
-                new_file.flush()
-                features = load_features(new_file.name, verbose=False)
             elif type == "gtf":
+                new_file = NamedTemporaryFile(suffix=".gtf")
                 new_file.write(data2gff(data))
-                new_file.flush()
-                features = load_features(new_file.name, gtf=True, sort=True,
-                                         verbose=False)
+
+            new_file.flush()
+            self._open_files.append(new_file)
+            features = Annotations(new_file.name, verbose=False)
 
             if features:
                 self.features.append(features)
@@ -307,4 +306,4 @@ def suite():
     return unittest.TestSuite(tests)
 
 if __name__ == "__main__":
-    unittest.main()
+    unittest.TextTestRunner(verbosity=2).run(suite())
